<?php
	// vim:ts=2 sts=2 sw=2
	// $Id: sakurascript.inc.php 7 2006-03-25 19:58:29Z fifthmoon $

	/*
	* PukiWiki SakuraScript plugin
	* Copyright (C) 2005,2006 fifth moon <fifthmoon+pukiwiki-plugin@tarenari.jp>
	* Original design: JavaScirpt written by ukiya. <ukiya at saku2 dot com>
	* SVG concept by TJ <Joe at ps dot sakura dot ne dot jp>
	*
	* ChangeLog:
	*   v1.1
	*     - Ignore newline tag when first text.
	*     - Change html selector.
	*         move scope_? from id to sstr's class
	*         add ghost_name to sstr's class
	*         add surface_? to sstr's class
	*   v1.0
	*     - first release.
	*/

	define('PLUGIN_SAKURASCRIPT_GHOST_DIR', IMAGE_DIR . 'ghost/');
	define('PLUGIN_SAKURASCRIPT_DEFAULT_GHOST', 'emily4');
	define('PLUGIN_SAKURASCRIPT_DEFAULT_0_SURFACE', 0);
	define('PLUGIN_SAKURASCRIPT_DEFAULT_1_SURFACE', 10);
	define('PLUGIN_SAKURASCRIPT_DEFAULT_SURFACE', 10);
	define('PLUGIN_SAKURASCRIPT_DEFAULT_SCOPE', 0);
	define('PLUGIN_SAKURASCRIPT_SURFACE_PREFIX', 'surface');
	define('PLUGIN_SAKURASCRIPT_SURFACE_IMAGE_TYPE', 'png');
	define('PLUGIN_SAKURASCRIPT_SURFACE_IMAGE_HEIGHT', 64);
	define('PLUGIN_SAKURASCRIPT_SURFACE_IMAGE_WIDTH', 48);
	define('PLUGIN_SAKURASCRIPT_SURFACE_NONE_IMAGE', 'none');
	define('PLUGIN_SAKURASCRIPT_REVERSE_SYNC', false);

	class parse_object {
		var $ghost_;
		var $scope_;
		var $text_;
		var $surface_;
		var $sync_;
		var $html_;

		function __construct() {
			$this->ghost_ = PLUGIN_SAKURASCRIPT_DEFAULT_GHOST;
			$this->scope_ = PLUGIN_SAKURASCRIPT_DEFAULT_SCOPE;
			$this->text_  = '';
			$this->surface_ = array(PLUGIN_SAKURASCRIPT_DEFAULT_0_SURFACE, PLUGIN_SAKURASCRIPT_DEFAULT_1_SURFACE);
			$this->sync_ = 0;
			$this->html_  = '';
		}
		function parse_object() {
			$this->__construct();
		}

		function set_ghost($_ghost) {
			if ($this->ghost_ != $_ghost) {
				$this->convert_table();
			}
			$this->ghost_ = $_ghost;
			$this->scope_ = PLUGIN_SAKURASCRIPT_DEFAULT_SCOPE;
			$this->surface_ = array(PLUGIN_SAKURASCRIPT_DEFAULT_0_SURFACE, PLUGIN_SAKURASCRIPT_DEFAULT_1_SURFACE);
			$this->sync_ = 0;
		}

		function set_surface($_surface) {
			if ($this->surface_[$this->scope_] != $_surface) {
				$this->convert_table();
			}
			$this->surface_[$this->scope_] = $_surface;
		}

		function set_scope($_scope) {
			if ($this->scope_ != $_scope) {
				$this->convert_table();
			}
			$this->scope_ = $_scope;
		}

		function set_sync($_scope) {
			$this->convert_table();
			$this->sync_ = !$this->sync_;
		}

		function set_newline() {
			if (mb_strlen($this->text_)) {
				$this->set_string("\n");
			}
		}

		function set_string($_ch) {
			$this->text_ .= $_ch;
		}

		function set_url($_url, $_str) {
			$_matches = array();
			if (preg_match("/href=\"(.+)\" /", $_url, $_matches)) {
				$_url = $_matches[1];
				if (!mb_strlen($_str)) {
					$_str = $_url;
				}
				if (is_url($_url)) {
					$this->text_ .= '<a href="' . $_url . '">' . $_str . '</a>';
				}
			}
		}

		function get_table() {
			$this->convert_table();
			return <<<EOT
<div class="sakurascript">
 <table class="sstable">
  <tbody>
$this->html_
  </tbody>
 </table>
</div>

EOT;
		}

		/* private */
		function surface_number($_number) {
			return sprintf('%04d', $_number);
		}

		function surface_file_name($_scope) {
			$_number = $this->surface_[$_scope] >= 0 ? $this->surface_number($this->surface_[$_scope]) : PLUGIN_SAKURASCRIPT_SURFACE_NONE_IMAGE;

			$_rtn = PLUGIN_SAKURASCRIPT_SURFACE_PREFIX
			. $_number
			. '.' . PLUGIN_SAKURASCRIPT_SURFACE_IMAGE_TYPE;

			return $_rtn;
		}

		function tag_img($_dir, $_imgs/* array*/) {
			$_rtn = '';
			foreach($_imgs as $_img) {
				$_rtn .= '<img src="' . $_dir . $_img
				. '" height="' . PLUGIN_SAKURASCRIPT_SURFACE_IMAGE_HEIGHT
				. '" width="'  . PLUGIN_SAKURASCRIPT_SURFACE_IMAGE_WIDTH
				. '" alt="'. $_img . '" />';
			}

			return $_rtn;
		}

		function tag_tr($_scope, $_dir, $_text, $_imgs/*array*/) {
			$number = $this->surface_number($this->surface_[$_scope]);
			$image = $this->tag_img($_dir, $_imgs);

			return <<<EOT
     <tr class="sstr scope_$_scope ghost_$this->ghost_ surface_$number">
      <td class="ssimage">
       $image
      </td>
      <td class="sstext">
       $_text
      </td>
     </tr>

EOT;
		}

		function convert_table() {
			if (mb_strlen($this->text_) != 0) {
				$this->text_ = nl2br(trim($this->text_));
				if (!array_key_exists($this->scope_, $this->surface_)) {
					$this->surface_[$this->scope_] = PLUGIN_SAKURASCRIPT_DEFAULT_SURFACE;
				}
				$_dir = PLUGIN_SAKURASCRIPT_GHOST_DIR . $this->ghost_ . '/';
				if ($this->sync_) {
					if (!PLUGIN_SAKURASCRIPT_REVERSE_SYNC) {
						$_p0 = $this->surface_file_name(0);
						$_p1 = $this->surface_file_name(1);
					} else {
						$_p0 = $this->surface_file_name(1);
						$_p1 = $this->surface_file_name(0);
					}
					$this->html_ .= $this->tag_tr('s', $_dir, $this->text_, array($_p0, $_p1));
				} else {
					$_p = $this->surface_file_name($this->scope_);
					$this->html_ .= $this->tag_tr($this->scope_, $_dir, $this->text_, array($_p));
				}
				$this->text_ = '';
			}
		}
	}

	function plugin_sakurascript_inline() {
		$_args = func_get_args();
		$_text = array_pop($_args); // htmlspecialchars(text)
		$_ghost = array_pop($_args); // {ghost} always exists

		$_po = new parse_object;
		$_po->set_ghost($_ghost ? $_ghost : PLUGIN_SAKURASCRIPT_DEFAULT_GHOST);

		for ($_i = 0; $_i < mb_strlen($_text); ++$_i) {
			$_ch = mb_substr($_text, $_i, 1);
			if ($_ch == '\\') {
				$_ch = mb_substr($_text, ++$_i, 1);
				switch($_ch) {
					case '0':
					case 'h':
						$_po->set_scope(0);
						break;
					case '1':
					case 'u':
						$_po->set_scope(1);
						break;
					case 'p':
						if (get_number($_text, $_i, $_no)) {
							$_po->set_scope($_no);
						} else {
							$_po->set_string('\\p' . $_no);
						}
						break;
					case 's':
						if (get_number($_text, $_i, $_no)) {
							$_po->set_surface($_no);
						} else {
							$_po->set_string('\\s' . $_no);
						}
						break;
					case 'w':
						$_ch = mb_substr($_text, ++$_i, 1);
						if (!ctype_digit($_ch)) {
							$_po->set_string('\\w' . $_ch);
						} // else { // ignore }
						break;
					case '_':
						$_ch = mb_substr($_text, ++$_i, 1);
						switch($_ch) {
							case 'w':
								if (!get_number($_text, $_i, $_no)) {
									$_po->set_string('\\_w' . $_no);
								}
								break;
							case 's':
								$_po->set_sync();
								break;
							case 'q':
								// ignore
								break;
						}
						break;
					case 'n':
						if (mb_substr($_text, $_i + 1, 1) == '[') {
							if (get_string($_text, $_i, $_str)) {
								$_po->set_newline();
							} else {
								$_po->set_string('\\n' . $_str);
							}
						} else {
							$_po->set_newline();
						}
						break;
					case 'g':
						if (get_string($_text, $_i, $_str)) {
							$_po->set_ghost($_str);
						} else {
							$_po->set_string('\\g' . $_str);
						}
						break;
					case 'U':
						if (mb_substr($_text, $_i + 1, 2) == 'RL') {
							$_i += 2;
							if (get_string($_text, $_i, $_url)) {
								if (mb_substr($_text, $_i + 1, 1) == '[') {
									if (get_string($_text, $_i, $_str)) {
										$_po->set_url($_url, $_str);
									} else {
										$_po->set_url($_url);
										$_po->set_string($_str);
									}
								} else {
									$_po->set_url($_url);
								}
							} else {
								$_po->set_string('\\URL' . $_url);
							}
						} // else { // ignore }
						break;
					case '\\':
						$_po->set_string($_ch);
						break;
					case 't':
					case 'c':
						// ignore
						break;
					case 'e':
						// ignore and break loop
						$_i = mb_strlen($_text);
						break;
					default:
						$_po->set_string('\\' . $_ch);
						break;
				}
			} else {
				$_po->set_string($_ch);
			}
		}

		return $_po->get_table();
	}

	function get_number(&$_text, &$_i, &$_no) {
		if (!get_string($_text, $_i, $_no)) {
			return false;
		}
		if (!is_numeric($_no)) {
			return false;
		}

		return true;
	}

	function get_string(&$_text, &$_i, &$_str) {
		$_str = '';
		$_ch = mb_substr($_text, ++$_i, 1);
		if ($_ch == '[') {
			for (++$_i; $_i < mb_strlen($_text); ++$_i) {
				$_ch = mb_substr($_text, $_i, 1);
				if ($_ch == ']') {
					return true;
				} else {
					$_str .= $_ch;
				}
			}
			if (mb_strlen($_str)) {
				$_str = '[' . $_str;
				return false;
			}
		} else {
			$_str = '[' . $_ch;
			return false;
		}
	}
?>
